const Fs = require("fs");
const Request = require('request');
const Path = require("path");
const { DOMParser} = require('xmldom');

var mediationData = {};

module.exports = {

	loadMediationConfig: function(url, configPath, callback) {

		if (!Fs.existsSync(configPath)) {
			Fs.mkdirSync(configPath);
			Editor.log('Create config path');
		}
		Request({url:url,json:true}, function (error, response, body) {
		  if (!error && response.statusCode == 200) {
		  		Editor.log('Get sdk info json'); 
		  		var settings = parseProjectSettings(response.url,body,configPath);
			    mediationData = parseSdkInfo(body,configPath);
			    callback({'settings':settings,'mediationData':mediationData});

		  } else {
		  		callback({});
		  }
		});
	},

	downloadSdkXml: function(sdkName, configPath, callback) {
		var sdk = {};
		if (sdkName == 'OpenMediation') {

			sdk = mediationData['mediation'];
		} else {
			var adapters = mediationData['adapters'];
			sdk = adapters[sdkName];
		}
		var file = sdk['file'];
		var url = sdk['url'];
		var version = sdk['version'];
		Editor.log('downloadSdkXml ' + sdkName);
		if (file.length && url.length) {
		    let stream = Fs.createWriteStream(Path.join(configPath, file));
		    Request(url).pipe(stream).on("close", function (err) {
		        if (err) {
		        	Editor.log('Download ' + sdkName +' error: ' +err);
		        	callback(null);
		        } else {
		        		Editor.log('Download ' + sdkName +' xml done'); 
		        		var localVersion = getVersionFromXml(Path.join(configPath, file));
						var versionState = getVersionState(version,localVersion);
						sdk['localVersion'] = localVersion;
						sdk['versionState'] = versionState;
						callback(sdk);

		        }
		    });
		} else {
			Editor.log(sdkName + ' file or url is empty'); 
			callback(null);
		}

	},

	updateSettings: function(configPath, lastSettings) {
		var projectSettingPath = Path.join(configPath, "settings");
		var settings = {};
		if (Fs.existsSync(projectSettingPath)) {
			let rawdata = Fs.readFileSync(projectSettingPath);
			settings = JSON.parse(rawdata);
		}
		for (let key in lastSettings) {
			settings[key] = lastSettings[key];
  		}
		let data = JSON.stringify(settings);
		Fs.writeFileSync(projectSettingPath, data);

		Editor.log('updateSettings done');
	}
}




function parseSdkInfo (sdkInfo, configPath) {
	var mediationSdkData = {};
	var sdks = sdkInfo['SDKSInfo'];

	var adapters = {};
	for (var k in sdks) {
		var s = sdks[k];
		var n = s["keyname"];
		var adapter = s["Adapter"];
		var file = adapter["FileName"];
		var url = adapter["DownloadUrl"];
		var version = adapter["version"];
		var iosSdkV = ''
		if (s.hasOwnProperty("iOS")) {
			iosSdkV = s["iOS"]["version"]["sdk"];
		}	

		var androidSdkV = '';

		if (s.hasOwnProperty("Android")) {
			androidSdkV = s["Android"]["version"]["sdk"];
		}

		var localVersion = getVersionFromXml(Path.join(configPath, file));

		var versionState = getVersionState(version,localVersion);

		var sdk = {'name':n,'version':version,'localVersion':localVersion,'versionState':versionState,'file':file,'url':url,'iosSdkV':iosSdkV,'androidSdkV':androidSdkV};

		if (n.toLowerCase() == 'openmediation') {
			mediationSdkData['mediation'] = sdk;
		} else {
			adapters[n] = sdk;
		}
	}

	mediationSdkData['adapters'] = adapters;
	return mediationSdkData;
}


function parseProjectSettings(url, info, configPath) {
	var projectSettingPath = Path.join(configPath, "settings");
	var settings = {};
	if (Fs.existsSync(projectSettingPath)) {
		let rawdata = Fs.readFileSync(projectSettingPath);
		settings = JSON.parse(rawdata);
	}
	if (url.indexOf('cn')>0) {
		settings['cn'] = true;
	} else {
		settings['cn'] = false;		
	}
	var projectSettings = info['ProjectSettings'];
	var skIds = projectSettings['iOS']['SKAdNetworkIds'];
	settings['skIds'] = skIds;
	let data = JSON.stringify(settings);
	Fs.writeFileSync(projectSettingPath, data);
	Editor.log('Update SKAdNetwork ID'); 
	return settings;
}

function getVersionState (remoteVersion, localVersion) {
	if (!localVersion.length) {
		return -1; 
	} else if (remoteVersion === localVersion) {
		return 1;
	} else {
		return 0;
	}
}



function getVersionFromXml (xmlFile) {
	var version = '';
	if (Fs.existsSync(xmlFile)) {
		let content = Fs.readFileSync(xmlFile, 'utf8');
		let doc = new DOMParser().parseFromString(content,'text/xml');
        let versions = doc.documentElement.getElementsByTagName("version");
        if (versions.length>0) {
        	version = versions[0].childNodes[0].nodeValue;
        }
	}
	return version;
}


