const Fs = require("fs");
const Path = require("path");
const Xcode = require("xcode");
const ExecSync = require('child_process').execSync;

const { DOMParser,XMLSerializer} = require('xmldom');
// const xmlserializer = require('xmlserializer');
const Plist = require('simple-plist');


module.exports = {

	installOpenMediation: function(target, packagePath) {

		let platform = target.platform;
		let buildPath = Path.normalize(target.dest);
		let projectName = target.projectName;


		copyPluginFile(buildPath, packagePath);

		insertStrToFile({
				filePath: Path.join(buildPath, "frameworks", "runtime-src", "Classes", "AppDelegate.cpp"),
				checkStr: '#include "OpenMediationJSHelper.h"',
				positionStr: '#include "cocos2d.h"',
				before: 0,
				insertStr: '\n#include "OpenMediationJSHelper.h"\n\n'
			});

		insertStrToFile({
						filePath: Path.join(buildPath, "frameworks", "runtime-src", "Classes", "AppDelegate.cpp"),
						checkStr: "se->addRegisterCallback(register_all_CocosOMPluginJS);",
						positionStr: "    se->start();",
						before: 1,
						insertStr: "\n    se->addRegisterCallback(register_all_CocosOMPluginJS);\n\n"
						});
		
		Editor.log('Insert plugin code to AppDelegate.cpp');

		if (platform.toLowerCase() === 'android') {
			handleAndroid(buildPath,packagePath);
		} else if (platform.toLowerCase() === 'ios') {
			handleiOS(buildPath,packagePath);
		}

		return true;
	}
}

function insertStrToFile(event) {
	var data = Fs.readFileSync(event.filePath);
	if (!(data.indexOf(event.checkStr) >= 0)) {
		var insertIndex = data.indexOf(event.positionStr);
		if (insertIndex < 0) 
			console.log("fail to find " + event.positionStr);
		else {
			var buf = Buffer.alloc(data.length + event.insertStr.length);
			var offset = buf.write(data.toString("utf8", 0, insertIndex));
			if (event.before) {
				offset += buf.write(event.insertStr, offset);
				offset += buf.write(event.positionStr, offset);
			} else {
				offset += buf.write(event.positionStr, offset);
				offset += buf.write(event.insertStr,offset);
			}
			buf.write(data.toString("utf8", insertIndex + event.positionStr.length, data.length), offset);
			Fs.writeFileSync(event.filePath,buf);
		}
	}
}

function updateDependence(filePath, insertPostion, fullCommond, splitStr) {

	var data = Fs.readFileSync(filePath);
	if (data.indexOf(fullCommond) < 0) {
		var commondArray = fullCommond.split(splitStr);
		commondArray.pop();
		var moduleName = commondArray.join(splitStr);
		var moduleLineIndex = -1;
		if (data.indexOf(moduleName) >= 0) {
			var lineArray = data.toString().split('\n');
			for (var i = 0; i < lineArray.length; i++) {
				var line = lineArray[i];
				var t = line.trim().split(splitStr);
				t.pop();
				var m = t.join(splitStr);
				if (m === moduleName) {
					moduleLineIndex = i;
					break;
				}
			}
			if (moduleLineIndex >= 0) {
				var line = '    ' + fullCommond;
				lineArray.splice(moduleLineIndex,1,line);
				Fs.writeFileSync(filePath,lineArray.join('\n'));
			}

		} 
		if (moduleLineIndex == -1) {
			var insertStr = '\n    ' + fullCommond;

			insertStrToFile({
				filePath: filePath,
				checkStr: insertStr,
				positionStr: insertPostion,
				before: 0,
				insertStr: insertStr
			});

		}
	}
}



function copyPluginFile(buildPath, packagePath) {
	var classPath = Path.join(buildPath, "frameworks", "runtime-src", "Classes");
	var pluginPath = Path.join(packagePath,"plugin");

	Fs.readdirSync(pluginPath).forEach(function(file) {
		var filePath = Path.join(pluginPath,file);
		var stat = Fs.lstatSync(filePath);
		if (!stat.isDirectory() && !file.startsWith(".")) {
			Fs.copyFileSync(Path.join(pluginPath,file),Path.join(classPath,file));
		}
	});

	Editor.log('Copy plugin file to Cocos Classes dir ');
}


function handleAndroid(buildPath, packagePath) {
	copyAnroidBridge(buildPath,packagePath);
	addFileToMK(buildPath);
	setAndroidAdmobId(buildPath,packagePath);
	addAndroidDependence(buildPath,packagePath);

}

function copyAnroidBridge(buildPath,packagePath) {
	let libPath = Path.join(buildPath, "frameworks", "runtime-src", "proj.android-studio", "app", "libs");
	if (!Fs.existsSync(libPath)) {
		Fs.mkdirSync(libPath);
		Editor.log('Create Android libs');
	}

	let bridgeFilePath = Path.join(packagePath, "plugin", "Android", "CocosPlugin.aar");

	Fs.copyFileSync(bridgeFilePath,Path.join(libPath,"CocosPlugin.aar"));
	Editor.log('Copy Android bridge file');

}

function addFileToMK(buildPath) {
	let mkFile = Path.join(buildPath, "frameworks", "runtime-src", "proj.android-studio", "jni", "CocosAndroid.mk");
	let insertStr = '\n				   ../../Classes/OMCocosAndroidBridge.cpp \\\n				   ../../Classes/OpenMediationWrapper.cpp \\\n				   ../../Classes/OpenMediationJSHelper.cpp \\';

	insertStrToFile({
						filePath: mkFile,
						checkStr: 'OMCocosAndroidBridge.cpp',
						positionStr: '../../Classes/jsb_module_register.cpp \\',
						before: 0,
						insertStr: insertStr
					})

	Editor.log('Add file to CocosAndroid.mk');
}

function setAndroidAdmobId(buildPath,packagePath) {
	let manifest = Path.join(buildPath, "frameworks", "runtime-src", "proj.android-studio", "app", "AndroidManifest.xml");
	var projectSettingPath = Path.join(packagePath, "config", "settings");

	if (Fs.existsSync(projectSettingPath)) {
		let rawdata = Fs.readFileSync(projectSettingPath);
		let settings = JSON.parse(rawdata);
		let admobId = settings['androidAdmobId'];
		let cn =  settings['cn'];
		if (admobId != undefined && admobId.length >0) {
			if (!Fs.existsSync(manifest)) {
				Editor.error("AndroidManifest.xml is missing. Try to rebuild the project.");
				return;
			}

			let content = Fs.readFileSync(manifest,'utf8');
			let doc = new DOMParser().parseFromString(content,'text/xml');
            let elemApplication = doc.documentElement.getElementsByTagName("application");
            if (elemApplication.length <= 0) {
                Editor.error("AndroidManifest.xml is not valid. Try to rebuild the project. application");
                return;
            }

            let metaDatas = elemApplication[0].getElementsByTagName("meta-data");
            var lastMetaData = null;
            var hasAdMobIdMetaData = false;
            for (var i = 0; i < metaDatas.length; i++) {
            	let m = metaDatas[i];
            	lastMetaData = m;
            	if (m.getAttribute('android:name') == 'com.google.android.gms.ads.APPLICATION_ID') {
            		hasAdMobIdMetaData = true;
            		m.setAttribute('android:value',admobId);
            		Editor.log('Update AdMob ID ' + admobId);
            		break;
            	}
            }

            if (!hasAdMobIdMetaData) {
	            let newNode=doc.createElement("meta-data");
	            newNode.setAttribute('android:name','com.google.android.gms.ads.APPLICATION_ID');
	            newNode.setAttribute('android:value',admobId);
	            elemApplication[0].insertBefore(newNode,elemApplication[0].getElementsByTagName("activity")[0]);
	           	for (var i = 0; i < 2; i++) {
	           		let wrap=doc.createTextNode('\n        ');
	           		elemApplication[0].insertBefore(wrap,elemApplication[0].getElementsByTagName("activity")[0]);
	           	}
	           	let comment =doc.createComment('OpenMediation Integration Manager: AdMob App ID');
	           	elemApplication[0].insertBefore(comment,newNode);
	           	let commentWrap=doc.createTextNode('\n        ');
	            elemApplication[0].insertBefore(commentWrap,newNode);
	           	let split =doc.createTextNode('\n        ');
	           	elemApplication[0].insertBefore(split,comment);

	            Editor.log('Add AdMob ID ' + admobId);

            }

            var text = (new XMLSerializer()).serializeToString(doc);
		
            Fs.writeFileSync(manifest,text);


		} else if (Fs.existsSync(Path.join(packagePath, "config", "AdMobDependencies.xml")) && !cn) {
			Editor.error('Android AdMob app ID is empty. Please enter a valid app ID to run ads properly.');			
		}
	}



}


function addAndroidDependence(buildPath, packagePath) {

	let appGradle = Path.join(buildPath, "frameworks", "runtime-src", "proj.android-studio", "app", "build.gradle");

	let positionStr = "implementation project(':libcocos2dx')";

	// module
	var dependences = checkAndroidDependencesXml(packagePath);
	for (var str of dependences) {
		str = str.replace("@aar","");
		var fullCommond = "implementation '" + str +"'"; 
		updateDependence(appGradle,positionStr,fullCommond,":");

		Editor.log('add Android dependence ' + str );
	}

	let buildGradle = Path.join(buildPath, "frameworks", "runtime-src", "proj.android-studio", "build.gradle");
	var repositories = checkRepositoriesXml(packagePath);

	for (var str of repositories) {

	var fullCommond = 'maven {\n            url "'  + str + '"\n        }\n        ';


	insertStrToFile({
						filePath: buildGradle,
						checkStr: str,
						positionStr: 'flatDir',
						before: 1,
						insertStr: fullCommond
					})

		Editor.log('add Android repositorie ' + str );
	}


}

function checkAndroidDependencesXml(packagePath) {
	var dependences = new Array();
	let configPath = Path.join(packagePath, "config");
	var dependences = new Array();
	Fs.readdirSync(configPath).forEach(function(file) {
		if (file.endsWith(".xml")) {
			var strs = checkStr(Path.join(configPath,file),'<androidPackage spec="', '"><repositories>');
			dependences.push.apply(dependences, strs);

			let content = Fs.readFileSync(Path.join(configPath,file),'utf8');
			let doc = new DOMParser().parseFromString(content,'text/xml');
			var	moudle = doc.documentElement.getElementsByTagName('androidPackages');
			if (moudle.length > 0) {
				var packages = moudle[0].childNodes;
				for (var i=0;i<packages.length;i++) {
					dependences.push(packages[i].getAttribute('spec'));
				}
			}


		}
	});

	return dependences;
}


function checkRepositoriesXml(packagePath) {
	
	var dependences = new Array();
	let configPath = Path.join(packagePath, "config");
	var allRepositories = new Array();
	Fs.readdirSync(configPath).forEach(function(file) {
		if (file.endsWith(".xml")) {
			let content = Fs.readFileSync(Path.join(configPath,file),'utf8');
			let doc = new DOMParser().parseFromString(content,'text/xml');
			var	moudle = doc.documentElement.getElementsByTagName('androidPackages');
			if (moudle.length > 0) {
				var packages = moudle[0].childNodes;
				for (var i=0;i<packages.length;i++) {
					var repositories = packages[i].getElementsByTagName('repositories');
					if (repositories.length >0) {
							var repository = repositories[0].childNodes;

							for (var j = 0; j < repository.length; j++) {
								var r = repository[j].childNodes[0].nodeValue;
								if (allRepositories.indexOf(r) == -1) {
									allRepositories.push(r);
								}
							}
					}
				}
			}

		}
	});

	return allRepositories;
}


function handleiOS(buildPath, packagePath) {

		var targetName = addFileToTarget(buildPath);

		Editor.log('Add code file to Xcode target ' + targetName);

		setIosAdMobId(buildPath,packagePath);

		addSkId(buildPath,packagePath);

		addPodDependence(buildPath, targetName, packagePath);

		Editor.log('Add pod dependence');

}


function addFileToTarget(buildPath) {

	var iosPath = Path.join(buildPath, "frameworks", "runtime-src", "proj.ios_mac");
	var xcodeFile = "";

	Fs.readdirSync(iosPath).forEach(function(file) {
		if (file.endsWith("xcodeproj")) {
			xcodeFile = file;
			Editor.log('Xcode project file: ' + xcodeFile);
		}	
	});

	var xcodeProjectPath = Path.join(iosPath,xcodeFile,"project.pbxproj");

	var xcodeProject = Xcode.project(xcodeProjectPath);

	xcodeProject.parseSync();

	var targets = xcodeProject.pbxNativeTargetSection();

	var targetName = "";
	for (var t in targets) {
		var n = targets[t].name;
		if (n && n.endsWith('mobile"')) {
			targetName = n;
		}
	}
	var target = xcodeProject.findTargetKey(targetName);
	var classGroup = xcodeProject.findPBXGroupKey({
		name: "Classes"
	});

	xcodeProject.addHeaderFile("OMClass.h", {}, classGroup);
	xcodeProject.addHeaderFile("OMCocosiOSBridge.h", {}, classGroup);
	xcodeProject.addHeaderFile("OpenMediationWrapper.h", {}, classGroup);
	xcodeProject.addHeaderFile("OpenMediationJSHelper.h", {}, classGroup);


	xcodeProject.addSourceFile("OMCocosiOSBridge.mm", {target:target, compilerFlags:"-fobjc-arc"}, classGroup);
	xcodeProject.addSourceFile("OpenMediationWrapper.cpp", {target: target}, classGroup);
	xcodeProject.addSourceFile("OpenMediationJSHelper.cpp", {target: target}, classGroup);

	let INHERITED = '"$(inherited)"';
    var config = xcodeProject.pbxXCBuildConfigurationSection();
    for (var ref in config) {
        if (ref.indexOf('_comment') == -1  && config[ref].buildSettings.SDKROOT == 'iphoneos' ) {
        	var buildSettings = config[ref].buildSettings;
        	var searchPaths = buildSettings.LIBRARY_SEARCH_PATHS;
        	if (!searchPaths) {
        		buildSettings.LIBRARY_SEARCH_PATHS = [];
        	} else if(typeof searchPaths === 'string') {
        		buildSettings.LIBRARY_SEARCH_PATHS = [searchPaths];
        	}

        	if(Array.isArray(buildSettings.LIBRARY_SEARCH_PATHS) && !buildSettings.LIBRARY_SEARCH_PATHS.includes(INHERITED)) {
        		buildSettings.LIBRARY_SEARCH_PATHS.push(INHERITED);
        	}
        }
    }

	Fs.writeFileSync(xcodeProjectPath, xcodeProject.writeSync());

	return targetName;

}

function setIosAdMobId(buildPath, packagePath) {
	var infoPath = Path.join(buildPath, "frameworks", "runtime-src", "proj.ios_mac", "ios", "Info.plist");
	var data = Plist.readFileSync(infoPath);

	var projectSettingPath = Path.join(packagePath, "config", "settings");

	if (Fs.existsSync(projectSettingPath)) {
		let rawdata = Fs.readFileSync(projectSettingPath);
		let settings = JSON.parse(rawdata);
		let admobId = settings['iosAdmobId'];
		if (admobId != undefined && admobId.length >0) {
			data['GADApplicationIdentifier'] = admobId;
			Editor.log('Set AdMob ID ' + admobId);
		} else if (Fs.existsSync(Path.join(packagePath, "config", "AdMobDependencies.xml"))) {
			Editor.error('iOS AdMob app ID is empty. Please enter a valid app ID to run ads properly.');			
		}
	}

	Plist.writeFileSync(infoPath, data);
}

function addSkId(buildPath, packagePath) {
	var currentSkIds = new Array();
	var infoPath = Path.join(buildPath, "frameworks", "runtime-src", "proj.ios_mac", "ios", "Info.plist");
	var data = Plist.readFileSync(infoPath);
	var items = data['SKAdNetworkItems'];
	if (items != undefined) {
		for (var item of items) {
			currentSkIds.push(item['SKAdNetworkIdentifier']);
		}
	} else {
		items = new Array();
	}

	var projectSettingPath = Path.join(packagePath, "config", "settings");

	if (Fs.existsSync(projectSettingPath)) {
		let rawdata = Fs.readFileSync(projectSettingPath);
		let settings = JSON.parse(rawdata);
		let skIds = settings['skIds'];
		for (var id of skIds) {
			if (currentSkIds.indexOf(id) == -1) {
				items.push({'SKAdNetworkIdentifier':id});
				Editor.log('Add SKAdNetwork id: ' + id);
			}
		}
	}

	data['SKAdNetworkItems'] = items;

	Plist.writeFileSync(infoPath, data);
}


function addPodDependence(buildPath, targetName, packagePath) {

	var pods = checkDependencesXml(packagePath);	

	if (pods.length >0 ) { //check
		
		let target = targetName.replace(new RegExp("\"","gm"),"");
		let podDir = Path.join(buildPath, "frameworks", "runtime-src", "proj.ios_mac");
		let podFilePath = Path.join(podDir, "Podfile");

	    if (!Fs.existsSync(podFilePath)) {
	        Editor.log('Pod file not found, init it');
	        var log = ExecSync('pod init', {cwd: podDir});

	        Editor.log('CocosPods:'+log.toString());
	    }

	    // module
	    let positionStr = '# Pods for ' + target;

	    for (var pod of pods) {
	    	let moduleName = pod['name'];
	    	let version = pod['version'];

	    	if (typeof(moduleName)!="undefined" && moduleName && moduleName.length>0) {

				var fullCommond = "pod '" + moduleName + "'";

				if (typeof(version)!="undefined" && version && version.length>0) {
					fullCommond = fullCommond + " , '" + version + "'";
				}

				Editor.log('Add dependence ' + fullCommond);

				updateDependence(podFilePath,positionStr,fullCommond,",");
	    	}
	    }

		podInstall(buildPath);

		addIncludeToUserConfig(buildPath,target,"debug");
		addIncludeToUserConfig(buildPath,target,"release");
	}
}


function checkDependencesXml(packagePath) {
	var dependences = new Array();
	let configPath = Path.join(packagePath, "config");
	var podDependences = new Array();
	Fs.readdirSync(configPath).forEach(function(file) {
		if (file.endsWith(".xml")) {
			let content = Fs.readFileSync(Path.join(configPath,file),'utf8');
			let doc = new DOMParser().parseFromString(content,'text/xml');
			var	podsMoudle = doc.documentElement.getElementsByTagName('iosPods');
			if (podsMoudle.length > 0) {
				var pods = podsMoudle[0].childNodes;
				for (var i=0;i<pods.length;i++) {
					var pod = {};
					pod['name'] = pods[i].getAttribute('name');
					pod['version'] = pods[i].getAttribute('version');
					podDependences.push(pod);
				}
			}
		}
	});

	return podDependences;

}



function parsePod (podStrs) {
	var pods = new Array();
	for (var str of podStrs) {
		str = str.replace(/"/g,"'");
		var pod = {};
		var commond = str.split(' ');
		for (var p of commond) {
			var keyValue = p.split('=');
			if (keyValue.length == 2) {
				pod[keyValue[0]] = keyValue[1];
			}
		}
		pods.push(pod);
	}

	return pods;
} 


function checkStr(file, startStr, endStr) {
	var checkStrings = new Array();
	let content = Fs.readFileSync(file, 'utf8');
	let startIndex = content.indexOf(startStr);
	let endIndex = content.indexOf(endStr);
	while((startIndex > 0) && ((endIndex - startIndex) > 0)) {
		var str = content.substring((startIndex + startStr.length),endIndex);
		 checkStrings.push(str);
		startIndex = content.indexOf(startStr,startIndex+1);
		endIndex = content.indexOf(endStr,endIndex+1);
	}

	return checkStrings;
}


function addIncludeToUserConfig(buildPath, targetName, mode) {

	let path = Path.join(buildPath, "frameworks", "runtime-src", "proj.ios_mac", "ios", `UserConfigIOS.${mode}.xcconfig`);

    if (!Fs.existsSync(path)) {
        Editor.warn('file not found ', path);
        return;
    }
    let str = `#include "Pods/Target Support Files/Pods-${targetName}/Pods-${targetName}.${mode}.xcconfig"`;
    let content = Fs.readFileSync(path, 'utf8');

    if (content.indexOf(str) !== -1) {
        return;
    }

    content += str + "\n";
    Fs.writeFileSync(path, content);

}


function podInstall(buildPath) {
	let podDir = Path.join(buildPath, "frameworks", "runtime-src", "proj.ios_mac");
	let podFilePath = Path.join(podDir, "Podfile");

    if (!Fs.existsSync(podFilePath)) {
        Editor.warn('Pod file not found');
    } else {
    	var log = ExecSync('pod install', {cwd: podDir});
    	Editor.log('CocosPods:'+log.toString());
    }
}
