'use strict';

const {
    remote,
    dialog,
    ipcMain,
    BrowserWindow,
    globalShortcut
} = require('electron');

const Path = require("fire-path");
const installer = require('./install');

const config = require('./config')

var isCN = false;

module.exports = {

    load: function() {

        Editor.log('OpenMediation Plugin load');
    },

    unload: function() {
        Editor.log('OpenMediation Plugin unload');
    },

    messages: {
        'manager': function() {
            Editor.log('OpenMediation Integration Manager');
			isCN = false;
            createManagerWin('OpenMediation Integration Manager');
        },
        'managerCN': function() {
            Editor.log('OpenMediation Integration Manager(CN)');
			isCN = true;
            createManagerWin('OpenMediation Integration Manager(CN)');
        },
        'editor:build-finished': function(event, target) {
            Editor.log('Import Cocos OpenMediation Plugin');
            const packagePath = __dirname;
            let success = false;
            try {
                success = installer.installOpenMediation(target, packagePath);
            } catch(err) {
                Editor.log(err);
                success = false;
            }
            if (success) {
                Editor.log('Import Cocos Creator OpenMediation success');
            } else {
                Editor.log('Import Cocos Creator OpenMediation failed');
            }
        },
        'editor:build-start': function(event, target) {
        },
        'updateConfig': function() {
        }
    },
};


function createManagerWin(title) {
    var win = new BrowserWindow({
        width: 800,
        height: 600,
        show:false,
        resizable: false,
        alwaysOnTop:true,
        webPreferences: {
            nodeIntegration: true,
            enableRemoteModule: true,
        },
        title:title
    });
    win.title = title;
    win.once('ready-to-show', () => {
      win.show()
    })

    win.on('closed', () => {
        Editor.log('OpenMediation Manager closed');
        ipcMain.removeAllListeners('loadConfig');
        ipcMain.removeAllListeners('updateSDK');
        win = null
    })

    ipcMain.on('loadConfig', function(event) {

        let url = 'https://dl.openmediation.com/PmDependencies/OpenMediationSDKInfo.json';
        if (isCN) {
            url ='https://dl.openmediation.com/PmDependencies/cn/OpenMediationSDKInfo.json';
        }

        let configPath = Path.join(__dirname, 'config');
        Editor.log('Load Config data'); 
        config.loadMediationConfig(url,configPath, function(config) {
            Editor.log('Parse medaition data ' + config); 
            event.sender.send('loadMediationData', config);
        } );
    });

    ipcMain.on('updateSDK', function(event, name) {
        Editor.log('Download sdk xml ' + name); 
        config.downloadSdkXml(name, Path.join(__dirname, 'config'), function(sdk) {
            if (sdk)
            {
                Editor.log('Update ' + name +' UI');
                event.sender.send('updateSdkUI', sdk); 
            }　
        });
    });

    ipcMain.on('updateSettings', function(event, settings) {
        Editor.log('Update settings' + settings.toString()); 
        let configPath = Path.join(__dirname, 'config');
        config.updateSettings(configPath,settings);
    });

    win.loadURL(`file://${Path.join(__dirname, 'pages')}/index.html`);
}

