const os = require('os');
const path = require('path');
const fs = require('fs');

const {
	remote,
    shell,
    ipcRenderer,
} = require('electron');

var settings = null;

$(document).ready(() => {
    console.log('Document ready loadConfig...');   
    ipcRenderer.send('loadConfig');
    ipcRenderer.on('loadMediationData', function (event, config) {
    	console.log('loadConfig callback'); 
        settings = config['settings'];
    	loadMediationData(config['mediationData']);
        //loadSettings(settings);

    });

    ipcRenderer.on('updateSdkUI', function (event, sdk) {
    	hideDialog();
    	let name = sdk['name'];
    	console.log('updateSdkUI callback ' + name); 
    	if (name == 'OpenMediation') {
    		updateSdkUI($('.mediation-div').find('.content'),sdk);
    	} else {
    		let div = $('.adapter-div').find(`div[name="${name}"]`);
    		updateSdkUI(div,sdk);
    	}
    });
});

function loadMediationData(mediatonData) {


    let sdk = mediatonData['mediation'];
    updateSdkUI($('.mediation-div').find('.content'),sdk);

    let adapters = mediatonData['adapters'];

    for (var k in adapters) {
    	 let value = adapters[k];
    	 let name = value['name'];
    	  let adapterDiv = $(`*[name="${name}"]`);
    	  if (!adapterDiv.length) {
	    	 adapterDiv = $('.mediation-div').find('.content').clone();
	    	 updateSdkUI(adapterDiv,value);
	    	 adapterDiv.appendTo($('.adapter-div'));
    	  }
    }


	$(".button").click(function(){
	  var name = $(this).attr("name");
	  updateSdkXml(name);

	});

	$('#errorMessage').hide();
    $('.mediation-div').show();
    $('.adapter-div').show();

 
}


function loadSettings(settings) {
    let cn = settings['cn'];
    let androidAdmobId = settings['androidAdmobId'];
    let iosAdmobId = settings['iosAdmobId'];
    androidAdmobId = (androidAdmobId != undefined)?androidAdmobId:'';
    iosAdmobId = (iosAdmobId != undefined)?iosAdmobId:'';

    if ((cn && (iosAdmobId.length == 0)) || (!cn && (iosAdmobId.length == 0 || androidAdmobId.length == 0))) {
        $('#admobIdTitle').css('color','red');
    }
    if (androidAdmobId != undefined) {
        $('#androidAdmobId').val(androidAdmobId);
    }
    if (iosAdmobId != undefined) {
        $('#iosAdmobId').val(iosAdmobId);
    }

    $('#androidAdmobId').bind('input propertychange', function() { 
        updateSettings();
    });

    $('#iosAdmobId').bind('input propertychange', function() { 
        updateSettings();
    });
}

function updateSettings() {
        let androidAdmobId = $("#androidAdmobId").val();
        let iosAdmobId = $("#iosAdmobId").val();
        let cn = settings['cn'];
        if ((cn && (iosAdmobId.length == 0)) || (!cn && (iosAdmobId.length == 0 || androidAdmobId.length == 0))) {
            $('#admobIdTitle').css('color','red');
        } else {
            $('#admobIdTitle').css('color','black');
        }
        console.log('androidAdmobId ' + androidAdmobId +'\niosAdmobId ' + iosAdmobId);
        ipcRenderer.send('updateSettings',{'androidAdmobId':androidAdmobId,'iosAdmobId':iosAdmobId});
}



function updateSdkUI(eleUI, sdk) {
    let name = sdk['name'];
    let localVersion =  sdk['localVersion'];
    let version = sdk['version'];
    let versionState =  sdk['versionState'];

    eleUI.attr('name',name);

    eleUI.children(':eq(0)').text(name);
    eleUI.children(':eq(1)').text((localVersion.length>0)?localVersion:'none');
    eleUI.children(':eq(2)').text(version);

    if (versionState == -1) {
    	eleUI.children(':eq(3)').find('.button').text('Install');
    	eleUI.children(':eq(3)').find('.button').removeAttr("disabled");
    } else if (versionState == 0) {
    	eleUI.children(':eq(3)').find('.button').text('Update'); 
    	eleUI.children(':eq(3)').find('.button').removeAttr("disabled");  	
    } else if (versionState == 1) {
    	eleUI.children(':eq(3)').find('.button').text('Updated');  
    	eleUI.children(':eq(3)').find('.button').attr("disabled","disabled");	
    }
    eleUI.children(':eq(3)').find('.button').attr('name',name); 

    if(name.toLowerCase() == 'admob' && (versionState>-1) ){
        if (!eleUI.children(':eq(0)').find('#admobIdBox').length) {
            var admobIdBox = createAdMobIdBox();
            admobIdBox.appendTo(eleUI.children(':eq(0)'));

        }
    }
}

function createAdMobIdBox (){

    let cn = settings['cn'];
    let androidAdmobId = settings['androidAdmobId'];
    let iosAdmobId = settings['iosAdmobId'];
    androidAdmobId = (androidAdmobId != undefined)?androidAdmobId:'';
    iosAdmobId = (iosAdmobId != undefined)?iosAdmobId:'';

    var admobIdBox=$('<details id = "admobIdBox"></details>');

    var summary = $('<summary id = "admobIdTitle">Set AdMob ID</summary>');
    summary.appendTo(admobIdBox);

    if ((cn && (iosAdmobId.length == 0)) || (!cn && (iosAdmobId.length == 0 || androidAdmobId.length == 0))) {
        summary.css('color','red');
    }

    var admobIdDiv=$('<div id = "admobDiv"></div>');

    var androidDiv = $('<div><label>Android</label></div>');

    var androidInput = $('<input id="androidAdmobId" type="text"/>');

    if (androidAdmobId != undefined) {
        androidInput.val(androidAdmobId);
    }

    androidInput.bind('input propertychange', function() { 
        updateSettings();
    });

    androidInput.appendTo(androidDiv);

    androidDiv.appendTo(admobIdDiv);


    var iosDiv = $('<div><label>iOS</label></div>');

    var iosInput = $('<input id="iosAdmobId" type="text"/>');

    if (iosAdmobId != undefined) {
        iosInput.val(iosAdmobId);
    }


    iosInput.bind('input propertychange', function() { 
        updateSettings();
    });

    iosInput.appendTo(iosDiv);

    iosDiv.appendTo(admobIdDiv);

    var tip = $('<p>AdMob App ID will look similar to this sample ID: ca-app-pub-3940256099942544~3347511713</p>');

    tip.appendTo(admobIdDiv);

    admobIdDiv.appendTo(admobIdBox);

    return admobIdBox;
}

function updateSdkXml(name) {
	console.log('updateSdkXml ' + name); 
	showDialog(name);
    ipcRenderer.send('updateSDK',name);
}

function showDialog(name) {
	if(name.indexOf('(') > 0) {
		name = name.substr(0,name.indexOf('('));
	}
	let msg = 'Downloading ' + name +' Dependencies.xml';
	$('#dialog').find('#dialogMsg').text(msg);
	$(".closeBtn").click(function(){
		hideDialog();

	});
    var dialog = document.querySelector("dialog");
	dialog.showModal();


}

function hideDialog() {
    var dialog = document.querySelector("dialog");
	dialog.close();
}