// Copyright 2020 ADTIMING TECHNOLOGY COMPANY LIMITED
// Licensed under the GNU Lesser General Public License Version 3

#import <Foundation/Foundation.h>
#import "OMClass.h"
NS_ASSUME_NONNULL_BEGIN


typedef NS_ENUM(NSInteger, OMInitEvent) {
    OMInitEventSuccess = 0,
    
    OMInitEventFailed
};

typedef NS_ENUM(NSInteger, OMAdEvent) {
    OMAdEventAdAvailable = 0,
    
    OMAdEventLoadSuccess,

    OMAdEventLoadFailed,

    OMAdEventShow,

    OMAdEventShowFailed,

    OMAdEventVideoStart,

    OMAdEventVideoEnd,

    OMAdEventClick,

    OMAdEventClose,

    OMAdEventReceiveReward,
};

typedef NS_ENUM(NSInteger, OMImpressionDataEvent) {
    OMImpressionDataEventSuccess = 0,
    
    OMImpressionDataEventFailed
};


typedef void(^initEventBlock)(OMInitEvent event, NSString *error);
typedef void(^adEventBlock)(OMAdFormat adFormat, OMAdEvent event, BOOL available, NSString* scene, NSString* placementId, NSString *error);
typedef void(^impressionDataBlock)(OMImpressionDataEvent event, NSString *value);

@interface OMCocosBridge : NSObject<OMInterstitialDelegate,OMRewardedVideoDelegate,OMBannerDelegate,OMCrossPromotionDelegate,OMSplashDelegate,OMImpressionDataDelegate>
@property (nonatomic, strong) NSMutableDictionary* bannerAdMap;
@property (nonatomic, strong) NSMutableDictionary* bannerPositionMap;
@property (nonatomic, strong) initEventBlock initBlock;
@property (nonatomic, strong) adEventBlock adBlock;
@property (nonatomic, strong) impressionDataBlock impressionDataBlock;
@property (nonatomic, strong) NSMutableDictionary* splashAdMap;

+ (instancetype)sharedInstance;
//interstitial
- (BOOL)interstitialIsReady;

- (void)showInterstitial;

- (void)showInterstitialWithScene:(NSString *)scene;

//video
- (BOOL)videoIsReady;

- (void)showVideo;

- (void)showVideoWithScene:(NSString *)scene;

- (void)showVideoWithExtraParams:(NSString *)scene extraParams:(NSString *)extraParams;

//CrossPromotion
- (BOOL)promotionAdIsReady;

- (void)showPromotionWithScene:(NSString*)scene width:(NSInteger)width height:(NSInteger)height scaleX:(CGFloat)scaleX scaleY:(CGFloat)scaleY  angle:(CGFloat)angle;

- (void)hidePromotionAd;

//Banner
- (void)loadBanner:(NSInteger)bannerType position:(NSInteger)position placement:(NSString *)placementID;

- (void)destroyBanner:(NSString*)placementID;

- (void)displayBanner:(NSString*)placementID;

- (void)hideBanner:(NSString*)placementID;

// Splash
- (void)loadSplashAd:(NSString *)placementID;

- (BOOL)splashAdIsReady:(NSString*)placementID;

- (void)showSplash:(NSString*)placementID;

@end

NS_ASSUME_NONNULL_END
