// Copyright 2020 ADTIMING TECHNOLOGY COMPANY LIMITED
// Licensed under the GNU Lesser General Public License Version 3

#include "platform/CCPlatformConfig.h"
#include <vector>
#if (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)

#import "OpenMediationWrapper.h"
#import "OMCocosiOSBridge.h"

#define OM_BANNER_POSITION_BOTTOM 0
#define OM_BANNER_POSITION_TOP 1


#define OMNSString2CString( str ) ( str != NULL && [str isKindOfClass:[NSString class]] ) ?  std::string([str UTF8String]) : std::string("")


#define OMCString2NSString( str )  [NSString stringWithUTF8String:str.c_str()]


namespace Cocos {

namespace OMPlugin {

void OpenMediationWrapper::nativeInit(std::string &appKey) {
    nativeSetAdBlock();
    NSString* nsAppKey = OMCString2NSString(appKey);
    Class omClass = NSClassFromString(@"OpenMediation");
    if (omClass && [omClass respondsToSelector:@selector(initWithAppKey:)]) {
        [omClass initWithAppKey:nsAppKey];
    }
    
}

void OpenMediationWrapper::nativeInit(std::string &appKey, std::string &host) {
    nativeSetAdBlock();
    NSString* nsAppKey = OMCString2NSString(appKey);
    NSString* nsHost = OMCString2NSString(host);;
    Class omClass = NSClassFromString(@"OpenMediation");
    if (omClass && [omClass respondsToSelector:@selector(initWithAppKey:baseHost:adFormat:)]) {
        [omClass initWithAppKey:nsAppKey baseHost:nsHost adFormat:(OMAdFormat)(OMAdFormatInterstitial|OMAdFormatRewardedVideo|OMAdFormatCrossPromotion)];
    }
}

void OpenMediationWrapper::nativeInit(std::string &appKey, std::string &host, std::string &channel) {
    nativeSetAdBlock();
    NSString* nsAppKey = OMCString2NSString(appKey);
    NSString* nsHost = OMCString2NSString(host);;
    Class omClass = NSClassFromString(@"OpenMediation");
    if (omClass && [omClass respondsToSelector:@selector(initWithAppKey:baseHost:adFormat:)]) {
        [omClass initWithAppKey:nsAppKey baseHost:nsHost adFormat:(OMAdFormat)(OMAdFormatInterstitial|OMAdFormatRewardedVideo|OMAdFormatCrossPromotion)];
    }
}

void OpenMediationWrapper::nativeSetAdBlock() {
    [OMCocosBridge sharedInstance].initBlock = ^(OMInitEvent event, NSString * _Nonnull error) {
        nativeInitCallBack((OpenMediationInitEvent)event,OMNSString2CString(error));
    };
    
    [OMCocosBridge sharedInstance].adBlock = ^(OMAdFormat adFormat, OMAdEvent event, BOOL available, NSString * _Nonnull scene, NSString * _Nonnull placementId, NSString * _Nonnull error) {
        nativeAdEventCallBack((OpenMediationAdFormat)adFormat,(OpenMediationAdEvent)event,available,OMNSString2CString(scene),OMNSString2CString(placementId),OMNSString2CString(error));
    };
    
    [OMCocosBridge sharedInstance].impressionDataBlock = ^(OMImpressionDataEvent event, NSString * _Nonnull value) {
        nativeImpressionDataCallBack((OpenMediationImpressionDataEvent)event,OMNSString2CString(value));
    };
}

bool OpenMediationWrapper::nativeIsInitialized() {
    bool initialized = false;
    Class omClass = NSClassFromString(@"OpenMediation");
    if (omClass && [omClass respondsToSelector:@selector(isInitialized)]) {
        initialized = [omClass isInitialized];
    }
    return initialized;
}



void OpenMediationWrapper::nativeSetIap(float count, std::string &currency) {
    NSString* curr = OMCString2NSString(currency);
    Class omClass = NSClassFromString(@"OpenMediation");
    if (omClass && [omClass respondsToSelector:@selector(userPurchase:currency:)]) {
        [omClass userPurchase:count currency:curr];
    }
}


void OpenMediationWrapper::nativeDebug(bool isDebug) {
    Class omClass = NSClassFromString(@"OpenMediation");
    if (omClass && [omClass respondsToSelector:@selector(setLogEnable:)]) {
        [omClass setLogEnable:isDebug];
    }
}

void OpenMediationWrapper::nativeSetGDPRConsent(bool consent) {
    Class omClass = NSClassFromString(@"OpenMediation");
    if (omClass && [omClass respondsToSelector:@selector(setGDPRConsent:)]) {
        [omClass setGDPRConsent:consent];
    }
}

void OpenMediationWrapper::nativeSetAgeRestricted(bool restricted) {
    Class omClass = NSClassFromString(@"OpenMediation");
    if (omClass && [omClass respondsToSelector:@selector(setUserAgeRestricted:)]) {
        [omClass setUserAgeRestricted:restricted];
    }
}

void OpenMediationWrapper::nativeSetUserAge(int age) {
    Class omClass = NSClassFromString(@"OpenMediation");
    if (omClass && [omClass respondsToSelector:@selector(setUserAge:)]) {
        [omClass setUserAge:age];
    }
}

void OpenMediationWrapper::nativeSetUserGender(OpenMediationUserGender userGender) {
    Class omClass = NSClassFromString(@"OpenMediation");
    if (omClass && [omClass respondsToSelector:@selector(setUserGender:)]) {
        [omClass setUserGender:(OMGender)userGender];
    }
}

void OpenMediationWrapper::nativeSetUSPrivacyLimit(bool limit) {
    Class omClass = NSClassFromString(@"OpenMediation");
    if (omClass && [omClass respondsToSelector:@selector(setUSPrivacyLimit:)]) {
        [omClass setUSPrivacyLimit:limit];
    }
}

bool OpenMediationWrapper::nativeGetGDPRConsent() {
    OMConsentStatus content = OMConsentStatusUnknown;
    Class omClass = NSClassFromString(@"OpenMediation");
    if (omClass && [omClass respondsToSelector:@selector(currentConsentStatus)]) {
        content = [omClass currentConsentStatus];
        if (content == OMConsentStatusDenied) {
            return NO;
        }
    }
    return YES;
}


void OpenMediationWrapper::nativeSetCustomTag(std::string &key, std::string &value) {
    NSString *keyString = OMCString2NSString(key);
    NSString *valueString = OMCString2NSString(value);
    Class omClass = NSClassFromString(@"OpenMediation");
    if (omClass && [omClass respondsToSelector:@selector(setCustomTag:withString:)]) {
        [omClass setCustomTag:keyString withString:valueString];
    }
}

void OpenMediationWrapper::nativeSetCustomTags(std::string &key, const std::vector<std::string> &values) {
    NSMutableArray * valueArray = [[NSMutableArray alloc] initWithCapacity: values.size()];
       for (std::string s : values)
       {
           [valueArray addObject: [NSString stringWithUTF8String: s.c_str()]];
       }
    Class omClass = NSClassFromString(@"OpenMediation");
    if (omClass && [omClass respondsToSelector:@selector(setCustomTag:withStrings:)]) {
        [omClass setCustomTag:OMCString2NSString(key) withStrings:valueArray];
    }
}

void OpenMediationWrapper::nativeRemoveCustomTag(std::string &key) {
    NSString *keyString = OMCString2NSString(key);
    Class omClass = NSClassFromString(@"OpenMediation");
    if (omClass && [omClass respondsToSelector:@selector(removeTag:)]) {
        [omClass removeTag:keyString];
    }
}


std::string OpenMediationWrapper::nativeGetCustomTags() {
    Class omClass = NSClassFromString(@"OpenMediation");
    if (omClass && [omClass respondsToSelector:@selector(allCustomTags)]) {
        NSDictionary *dic = [omClass allCustomTags];
        NSError *jsonError = nil;
        NSData *jsonData = [NSJSONSerialization dataWithJSONObject:dic options:NSJSONWritingPrettyPrinted error:&jsonError];
        NSString *jsonString = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
        return OMNSString2CString(jsonString);
    }
    return OMNSString2CString(@"");
}

void OpenMediationWrapper::nativeSetUserId(std::string &userId) {
    NSString *userIdString = OMCString2NSString(userId);
    Class omClass = NSClassFromString(@"OpenMediation");
    if (omClass && [omClass respondsToSelector:@selector(setUserID:)]) {
        [omClass setUserID:userIdString];
    }
}

std::string OpenMediationWrapper::nativeGetUserId() {
    Class omClass = NSClassFromString(@"OpenMediation");
    if (omClass && [omClass respondsToSelector:@selector(getUserID)]) {
        return OMNSString2CString([omClass getUserID]);;
    }
    return OMNSString2CString(@"");
}

bool OpenMediationWrapper::nativeInterstitialReady() {
    return [[OMCocosBridge sharedInstance] interstitialIsReady];
}

void OpenMediationWrapper::nativeShowInterstitial() {
    if ([[OMCocosBridge sharedInstance] interstitialIsReady])
    {
        [[OMCocosBridge sharedInstance] showInterstitial];
    }
}

void OpenMediationWrapper::nativeShowInterstitial(std::string &scene) {
    NSString* OmScene = OMCString2NSString(scene);
    if ([[OMCocosBridge sharedInstance] interstitialIsReady]) {
        [[OMCocosBridge sharedInstance] showInterstitialWithScene:OmScene];
    }
}




bool  OpenMediationWrapper::nativeRewardedVideoReady() {
    return [[OMCocosBridge sharedInstance] videoIsReady];
}

void  OpenMediationWrapper::nativeShowRewardedVideo() {
    if ([[OMCocosBridge sharedInstance] videoIsReady]){
        [[OMCocosBridge sharedInstance] showVideo];
    }
    
}


void  OpenMediationWrapper::nativeShowRewardedVideo(std::string &scene) {
    NSString* OmScene = OMCString2NSString(scene);
    if ([[OMCocosBridge sharedInstance] videoIsReady]) {
        [[OMCocosBridge sharedInstance] showVideoWithScene:OmScene];
    }
    
}

void OpenMediationWrapper::nativeShowRewardedVideo(std::string &scene, std::string &extraParams) {
    NSString* OmScene = OMCString2NSString(scene);
    NSString* OmParams = OMCString2NSString(extraParams);
    if ([[OMCocosBridge sharedInstance] videoIsReady]) {
        [[OMCocosBridge sharedInstance] showVideoWithExtraParams:OmScene extraParams:OmParams];
    }
    
}

//Banner

void  OpenMediationWrapper::nativeLoadBanner(std::string &placementId, OpenMediationAdSize adSize, OpenMediationBannerPosition position) {
    [[OMCocosBridge sharedInstance] loadBanner:(OMBannerType)adSize position:position placement:OMCString2NSString(placementId)];
}

void  OpenMediationWrapper::nativeDestroyBanner(std::string &placementId) {
    [[OMCocosBridge sharedInstance] destroyBanner:OMCString2NSString(placementId)];
}

void  OpenMediationWrapper::nativeDisplayBanner(std::string &placementId) {
    [[OMCocosBridge sharedInstance] displayBanner:OMCString2NSString(placementId)];
}

void  OpenMediationWrapper::nativeHideBanner(std::string &placementId) {
    [[OMCocosBridge sharedInstance] hideBanner:OMCString2NSString(placementId)];
}

// PromotionAd
bool  OpenMediationWrapper::nativePromotionAdReady() {
    return [[OMCocosBridge sharedInstance] promotionAdIsReady];
}

void  OpenMediationWrapper::nativeShowPromotionAd(std::string &scene, int width, int height, float scaleX, float scaleY, float angle) {
    NSString* OmScene = OMCString2NSString(scene);
    [[OMCocosBridge sharedInstance] showPromotionWithScene:OmScene width:width height:height scaleX:scaleX scaleY:scaleY angle: angle];
}


void OpenMediationWrapper::nativeHidePromotionAd() {
    [[OMCocosBridge sharedInstance] hidePromotionAd];
}

// Splash
void OpenMediationWrapper::nativeLoadSplash(std::string &placementId) {
    [[OMCocosBridge sharedInstance] loadSplashAd:OMCString2NSString(placementId)];
}

bool OpenMediationWrapper::nativeSplashAdReady(std::string &placementId) {
    return [[OMCocosBridge sharedInstance] splashAdIsReady:OMCString2NSString(placementId)];
}

void OpenMediationWrapper::nativeShowSplash(std::string &placementId) {
    [[OMCocosBridge sharedInstance] showSplash:OMCString2NSString(placementId)];
}
}
}


static OMCocosBridge * _instance = nil;

@implementation OMCocosBridge

+ (instancetype)sharedInstance{
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        _instance = [[self alloc] init];
    });
    return _instance;
}

- (instancetype)init{
    if (self = [super init]) {
        _bannerAdMap = [NSMutableDictionary dictionary];
        _bannerPositionMap = [NSMutableDictionary dictionary];
        _splashAdMap = [NSMutableDictionary dictionary];
        
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(initSuccess) name:@"kOpenMediatonInitSuccessNotification" object:nil];
        
        Class interstitialClass = NSClassFromString(@"OMInterstitial");
        if(interstitialClass && [interstitialClass respondsToSelector:@selector(sharedInstance)] && [interstitialClass instancesRespondToSelector:@selector(addDelegate:)]) {
            [[interstitialClass sharedInstance] addDelegate:self];
        }
        
        Class rewardedVideoClass = NSClassFromString(@"OMRewardedVideo");
        if(rewardedVideoClass && [rewardedVideoClass respondsToSelector:@selector(sharedInstance)] && [rewardedVideoClass instancesRespondToSelector:@selector(addDelegate:)]) {
            [[rewardedVideoClass sharedInstance] addDelegate:self];
        }
        
        Class promotionClass = NSClassFromString(@"OMCrossPromotion");
        if(promotionClass && [promotionClass respondsToSelector:@selector(sharedInstance)] && [promotionClass instancesRespondToSelector:@selector(addDelegate:)]) {
            [[promotionClass sharedInstance] addDelegate:self];
        }
        
        Class impressionClass = NSClassFromString(@"OpenMediation");
        if (impressionClass && [impressionClass respondsToSelector:@selector(addImpressionDataDelegate:)]) {
            [impressionClass addImpressionDataDelegate:self];
        }
        
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(orientationChanged:)
                                                     name:UIDeviceOrientationDidChangeNotification object:nil];
    }
    return self;
}

- (void)initSuccess {
    if (_initBlock) {
        _initBlock(OMInitEventSuccess,@"");
    }
}

- (BOOL)interstitialIsReady {
    BOOL isReady = NO;
    Class interstitialClass = NSClassFromString(@"OMInterstitial");
    if(interstitialClass && [interstitialClass respondsToSelector:@selector(sharedInstance)] && [interstitialClass instancesRespondToSelector:@selector(isReady)]) {
        isReady = [[interstitialClass sharedInstance] isReady];
    }
    return isReady;
}

- (void)showInterstitial {
    
    Class interstitialClass = NSClassFromString(@"OMInterstitial");
    if(interstitialClass && [interstitialClass respondsToSelector:@selector(sharedInstance)] && [interstitialClass instancesRespondToSelector:@selector(showWithViewController:scene:)]) {
        [[interstitialClass sharedInstance] showWithViewController:[UIApplication sharedApplication].delegate.window.rootViewController scene:@""];
    }
    
}

- (void)showInterstitialWithScene:(NSString *)scene {
    
    Class interstitialClass = NSClassFromString(@"OMInterstitial");
    if(interstitialClass && [interstitialClass respondsToSelector:@selector(sharedInstance)] && [interstitialClass instancesRespondToSelector:@selector(showWithViewController:scene:)]) {
        [[interstitialClass sharedInstance] showWithViewController:[UIApplication sharedApplication].delegate.window.rootViewController scene:scene];
    }
}

/// Invoked when a interstitial video is available.
- (void)omInterstitialChangedAvailability:(BOOL)available {
    if (_adBlock) {
        _adBlock(OMAdFormatInterstitial,OMAdEventAdAvailable,available,@"",@"",@"");
    }
}

/// Sent immediately when a interstitial video is opened.
- (void)omInterstitialDidOpen:(OMScene*)scene {
    if (_adBlock) {
        _adBlock(OMAdFormatInterstitial,OMAdEventShow,NO,scene.sceneName,@"",@"");
    }
}
/////////////
/// Sent immediately when a interstitial video starts to play.
- (void)omInterstitialDidShow:(OMScene*)scene {
    
    
}

/// Sent after a interstitial video has been clicked.
- (void)omInterstitialDidClick:(OMScene*)scene {
    if (_adBlock) {
        _adBlock(OMAdFormatInterstitial,OMAdEventClick,NO,scene.sceneName,@"",@"");
    }
    
}

/// Sent after a interstitial video has been closed.
- (void)omInterstitialDidClose:(OMScene*)scene {
    if (_adBlock) {
        _adBlock(OMAdFormatInterstitial,OMAdEventClose,NO,scene.sceneName,@"",@"");
    }
    
}

/// Sent after a interstitial video has failed to play.
- (void)omInterstitialDidFailToShow:(OMScene*)scene withError:(NSError *)error {
    if (error) {
        NSString *errorStr = [NSString stringWithFormat:@"error code:%ld msg:%@", (long)[error code],[error localizedDescription]];
        if (_adBlock) {
            _adBlock(OMAdFormatInterstitial,OMAdEventShowFailed,NO,scene.sceneName,@"",errorStr);
        }
        
    } else {
        if (_adBlock) {
            _adBlock(OMAdFormatInterstitial,OMAdEventShowFailed,NO,scene.sceneName,@"",@"");
        }
    }
}


#pragma mark -- video
- (BOOL)videoIsReady {
    BOOL isReady = NO;
    Class rewardedVideoClass = NSClassFromString(@"OMRewardedVideo");
    if(rewardedVideoClass && [rewardedVideoClass respondsToSelector:@selector(sharedInstance)] && [rewardedVideoClass instancesRespondToSelector:@selector(isReady)]) {
        isReady = [[rewardedVideoClass sharedInstance] isReady];
    }
    return isReady;
}

- (void)showVideo {
    Class rewardedVideoClass = NSClassFromString(@"OMRewardedVideo");
    if(rewardedVideoClass && [rewardedVideoClass respondsToSelector:@selector(sharedInstance)] && [rewardedVideoClass instancesRespondToSelector:@selector(showWithViewController:scene:)]) {
        [[rewardedVideoClass sharedInstance] showWithViewController:[[UIApplication sharedApplication] delegate].window.rootViewController scene:@""];
    }
}

- (void)showVideoWithScene:(NSString *)scene {
    Class rewardedVideoClass = NSClassFromString(@"OMRewardedVideo");
    if(rewardedVideoClass && [rewardedVideoClass respondsToSelector:@selector(sharedInstance)] && [rewardedVideoClass instancesRespondToSelector:@selector(showWithViewController:scene:)]) {
        [[rewardedVideoClass sharedInstance] showWithViewController:[[UIApplication sharedApplication] delegate].window.rootViewController scene:scene];
    }
}

- (void)showVideoWithExtraParams:(NSString *)scene extraParams:(NSString *)extraParams {
    Class rewardedVideoClass = NSClassFromString(@"OMRewardedVideo");
    if(rewardedVideoClass && [rewardedVideoClass respondsToSelector:@selector(sharedInstance)] && [rewardedVideoClass instancesRespondToSelector:@selector(showWithViewController:scene:extraParams:)]) {
        [[rewardedVideoClass sharedInstance] showWithViewController:[[UIApplication sharedApplication] delegate].window.rootViewController scene:scene extraParams:extraParams];
    }
    
}

#pragma mark -- OMRewardedVideoDelegate

- (void)omRewardedVideoChangedAvailability:(BOOL)available {
    
    if (_adBlock) {
        _adBlock(OMAdFormatRewardedVideo,OMAdEventAdAvailable,available,@"",@"",@"");
    }
}

- (void)omRewardedVideoDidOpen:(OMScene*)scene {
    if (_adBlock) {
        _adBlock(OMAdFormatRewardedVideo,OMAdEventShow,NO,scene.sceneName,@"",@"");
    }
    
}

- (void)omRewardedVideoPlayStart:(OMScene *)scene {
    if (_adBlock) {
        _adBlock(OMAdFormatRewardedVideo,OMAdEventVideoStart,NO,scene.sceneName,@"",@"");
    }
    
}

- (void)omRewardedVideoDidClick:(OMScene *)scene {
    if (_adBlock) {
        _adBlock(OMAdFormatRewardedVideo,OMAdEventClick,NO,scene.sceneName,@"",@"");
    }
}

- (void)omRewardedVideoDidClose:(OMScene *)scene{
    if (_adBlock) {
        _adBlock(OMAdFormatRewardedVideo,OMAdEventClose,NO,scene.sceneName,@"",@"");
    }
}

- (void)omRewardedVideoPlayEnd:(OMScene*)scene {
    if (_adBlock) {
        _adBlock(OMAdFormatRewardedVideo,OMAdEventVideoEnd,NO,scene.sceneName,@"",@"");
    }
}

- (void)omRewardedVideoDidReceiveReward:(OMScene*)scene {
    if (_adBlock) {
        _adBlock(OMAdFormatRewardedVideo,OMAdEventReceiveReward,NO,scene.sceneName,@"",@"");
    }
    
}

- (void)omRewardedVideoDidFailToShow:(OMScene *)scene withError:(NSError *)error {
    
    if (error) {
        NSString *errorStr = [NSString stringWithFormat:@"error code:%ld msg:%@", (long)[error code],[error localizedDescription]];
        if (_adBlock) {
            _adBlock(OMAdFormatRewardedVideo,OMAdEventShowFailed,NO,scene.sceneName,@"",errorStr);
        }
    } else {
        if (_adBlock) {
            _adBlock(OMAdFormatRewardedVideo,OMAdEventShowFailed,NO,scene.sceneName,@"",@"");
        }
    }
}


#pragma mark Banner API

- (void)loadBanner:(NSInteger)bannerType position:(NSInteger)position placement:(NSString *)placementId {
    @synchronized(self) {
        OMBanner *banner = [_bannerAdMap objectForKey:placementId];
        Class bannerClass =  NSClassFromString(@"OMBanner");
        if (!banner && bannerClass && [bannerClass instancesRespondToSelector:@selector(initWithBannerType:placementID:)]) {
            banner = [[bannerClass alloc]initWithBannerType:(OMBannerType)bannerType placementID:placementId];
            banner.delegate = self;
        }
        if (banner) {
            [_bannerAdMap setObject:banner forKey:placementId];
            [_bannerPositionMap setObject:[NSNumber numberWithInteger:position] forKey:placementId];
            banner.center = [self getBannerCenter:banner position:position];
            [[UIApplication sharedApplication].delegate.window.rootViewController.view addSubview:banner];
            [banner loadAndShow];
        }
        
        
    }
}

- (void)destroyBanner:(NSString*)placementId {
    dispatch_async(dispatch_get_main_queue(), ^{
        @synchronized(self) {
            OMBanner *banner = [_bannerAdMap objectForKey:placementId];
            if (banner != nil) {
                [banner removeFromSuperview];
            }
            [_bannerAdMap removeObjectForKey:placementId];
            [_bannerPositionMap removeObjectForKey:placementId];
        }
    });
}

- (void)displayBanner:(NSString*)placementId {
    dispatch_async(dispatch_get_main_queue(), ^{
        @synchronized(self) {
            OMBanner *banner = [_bannerAdMap objectForKey:placementId];
            if (banner != nil) {
                [banner setHidden:NO];
            }
        }
    });
}

- (void)hideBanner:(NSString*)placementId {
    dispatch_async(dispatch_get_main_queue(), ^{
        @synchronized(self) {
            OMBanner *banner = [_bannerAdMap objectForKey:placementId];
            if (banner != nil) {
                [banner setHidden:YES];
            }
        }
    });
}

- (CGPoint)getBannerCenter:(OMBanner*)bannerView position:(NSInteger)position {
    CGFloat y;
    UIView *topView = [UIApplication sharedApplication].keyWindow.rootViewController.view;
    if (position == OM_BANNER_POSITION_TOP) {
        y = (bannerView.frame.size.height / 2);
        if (@available(ios 11.0, *)) {
            y += topView.safeAreaInsets.top;
        }
    } else {
        y = topView.frame.size.height - (bannerView.frame.size.height / 2);
        if (@available(ios 11.0, *)) {
            y -= topView.safeAreaInsets.bottom;
        }
    }
    
    return CGPointMake(topView.frame.size.width / 2, y);
}

- (void)centerBanner {
    dispatch_async(dispatch_get_main_queue(), ^{
        @synchronized(self) {
            NSArray *placements = [_bannerAdMap allKeys];
            for (NSString *placementId in placements) {
                OMBanner *banner = [_bannerAdMap objectForKey:placementId];
                NSInteger position = [[_bannerPositionMap objectForKey:placementId]integerValue];
                banner.center = [self getBannerCenter:banner position:position];
            }
        }
    });
}

- (void)orientationChanged:(NSNotification *)notification {
    [self centerBanner];
}


#pragma mark Banner Delegate

- (void)omBannerDidLoad:(OMBanner *)banner {
    if (_adBlock) {
        _adBlock(OMAdFormatBanner,OMAdEventLoadSuccess,NO,@"",banner.placementID,@"");
    }
}


- (void)omBannerDidFailToLoad:(OMBanner *)banner withError:(NSError *)error {
    if (error) {
        NSString *errorStr = [NSString stringWithFormat:@"error code:%ld msg:%@", (long)[error code],[error localizedDescription]];
        if (_adBlock) {
            _adBlock(OMAdFormatBanner,OMAdEventLoadSuccess,NO,@"",banner.placementID,errorStr);
        }
    } else {
        if (_adBlock) {
            _adBlock(OMAdFormatBanner,OMAdEventLoadSuccess,NO,@"",banner.placementID,@"");
        }
    }
}


- (void)omBannerWillExposure:(OMBanner *)banner {
    if (_adBlock) {
        _adBlock(OMAdFormatBanner,OMAdEventShow,NO,@"",banner.placementID,@"");
    }
}


- (void)omBannerDidClick:(OMBanner *)banner {
    if (_adBlock) {
        _adBlock(OMAdFormatBanner,OMAdEventClick,NO,@"",banner.placementID,@"");
    }
}


- (void)omBannerWillPresentScreen:(OMBanner *)banner {
    //UnitySendMessage(kOMEvents, "onBannerScreenPresented", "");
}


- (void)omBannerDidDismissScreen:(OMBanner *)banner {
    //UnitySendMessage(kOMEvents, "onBannerScreenDismissed", "");
}


- (void)omBannerWillLeaveApplication:(OMBanner *)banner {
    //UnitySendMessage(kOMEvents, "onBannerLeftApplication", "");
}

- (BOOL)promotionAdIsReady {
    BOOL isReady = NO;
    Class promotionClass = NSClassFromString(@"OMCrossPromotion");
    if(promotionClass && [promotionClass respondsToSelector:@selector(sharedInstance)] && [promotionClass instancesRespondToSelector:@selector(isReady)]) {
        isReady = [[promotionClass sharedInstance] isReady];
    }
    return isReady;
}

- (void)showPromotionWithScene:(NSString*)scene width:(NSInteger)width height:(NSInteger)height scaleX:(CGFloat)scaleX scaleY:(CGFloat)scaleY  angle:(CGFloat)angle {
    
    Class promotionClass = NSClassFromString(@"OMCrossPromotion");
    if(promotionClass && [promotionClass respondsToSelector:@selector(sharedInstance)] && [promotionClass instancesRespondToSelector:@selector(showAdWithScreenPoint:adSize:angle:scene:)]) {
        [[promotionClass sharedInstance]showAdWithScreenPoint:CGPointMake(scaleX,scaleY) adSize:CGSizeMake(width,height) angle:angle scene:scene];
    }
    
}

- (void)hidePromotionAd {
    Class promotionClass = NSClassFromString(@"OMCrossPromotion");
    if(promotionClass && [promotionClass respondsToSelector:@selector(sharedInstance)] && [promotionClass instancesRespondToSelector:@selector(hideAd)]) {
        [[promotionClass sharedInstance]hideAd];
    }
}


- (void)omCrossPromotionChangedAvailability:(BOOL)available {
    if (_adBlock) {
        _adBlock(OMAdFormatCrossPromotion,OMAdEventAdAvailable,available,@"",@"",@"");
    }
}

- (void)omCrossPromotionWillAppear:(OMScene*)scene {
    if (_adBlock) {
        _adBlock(OMAdFormatCrossPromotion,OMAdEventShow,NO,scene.sceneName,@"",@"");
    }
}

- (void)omCrossPromotionDidClick:(OMScene*)scene {
    if (_adBlock) {
        _adBlock(OMAdFormatCrossPromotion,OMAdEventClick,NO,scene.sceneName,@"",@"");
    }
}

- (void)omCrossPromotionDidDisappear:(OMScene*)scene {
    if (_adBlock) {
        _adBlock(OMAdFormatCrossPromotion,OMAdEventClose,NO,scene.sceneName,@"",@"");
    }
}

- (void)omCrossPromotionDidFailToShow:(OMScene*)scene withError:(NSError *)error {
    if (error) {
        NSString *errorStr = [NSString stringWithFormat:@"error code:%ld msg:%@", (long)[error code],[error localizedDescription]];
        if (_adBlock) {
            _adBlock(OMAdFormatCrossPromotion,OMAdEventShowFailed,NO,scene.sceneName,@"",errorStr);
        }
    } else {
        if (_adBlock) {
            _adBlock(OMAdFormatCrossPromotion,OMAdEventShowFailed,NO,scene.sceneName,@"",@"");
        }
    }
}

#pragma mark Splash API
- (void)loadSplashAd:(NSString *)placementID {
    @synchronized(self) {
        OMSplash *splash = [_splashAdMap objectForKey:placementID];
        Class splashClass =  NSClassFromString(@"OMSplash");
        if (!splash && splashClass && [splashClass instancesRespondToSelector:@selector(initWithPlacementId:adSize:)]) {
            splash = [[splashClass alloc]initWithPlacementId:placementID adSize:CGSizeMake(375, 667)];
            splash.delegate = self;
        }
        if (splash) {
            [_splashAdMap setObject:splash forKey:placementID];
            [splash loadAd];
        }
        
    }
}

- (BOOL)splashAdIsReady:(NSString*)placementID {
    BOOL isReady = NO;
    OMSplash *splash = [_splashAdMap objectForKey:placementID];
    Class splashClass =  NSClassFromString(@"OMSplash");
    if (splash && splashClass && [splashClass respondsToSelector:@selector(isReady)]) {
        isReady = [splash isReady];
    }
    return isReady;
}

- (void)showSplash:(NSString*)placementID {
    OMSplash *splash = [_splashAdMap objectForKey:placementID];
    if (splash && [splash respondsToSelector:@selector(showWithWindow:customView:)]) {
        [splash showWithWindow:[UIApplication sharedApplication].keyWindow customView:nil];
    }
}


#pragma mark Splash Delegate

- (void)omSplashDidLoad:(OMSplash *)splash {
    if (_adBlock) {
        _adBlock(OMAdFormatSplash,OMAdEventLoadSuccess,NO,@"",splash.placementID,@"");
    }
}

- (void)omSplashFailToLoad:(OMSplash *)splash withError:(NSError *)error {
    if (error) {
        NSString *errorStr = [NSString stringWithFormat:@"error code:%ld msg:%@", (long)[error code],[error localizedDescription]];
        if (_adBlock) {
            _adBlock(OMAdFormatSplash,OMAdEventLoadSuccess,NO,@"",splash.placementID,errorStr);
        }
    } else {
        if (_adBlock) {
            _adBlock(OMAdFormatSplash,OMAdEventLoadSuccess,NO,@"",splash.placementID,@"");
        }
    }
}

- (void)omSplashDidShow:(OMSplash *)splash {
    if (_adBlock) {
        _adBlock(OMAdFormatSplash,OMAdEventShow,NO,@"",splash.placementID,@"");
    }
}

- (void)omSplashDidClick:(OMSplash *)splash {
    if (_adBlock) {
        _adBlock(OMAdFormatSplash,OMAdEventClick,NO,@"",splash.placementID,@"");
    }
}

- (void)omSplashDidClose:(OMSplash *)splash {
    if (_adBlock) {
        _adBlock(OMAdFormatSplash,OMAdEventClose,NO,@"",splash.placementID,@"");
    }
}

- (void)omSplashDidFailToShow:(OMSplash *)splash withError:(NSError *)error {
    if (error) {
        NSString *errorStr = [NSString stringWithFormat:@"error code:%ld msg:%@", (long)[error code],[error localizedDescription]];
        if (_adBlock) {
            _adBlock(OMAdFormatSplash,OMAdEventShowFailed,NO,@"",splash.placementID,errorStr);
        }
    } else {
        if (_adBlock) {
            _adBlock(OMAdFormatSplash,OMAdEventShowFailed,NO,@"",splash.placementID,@"");
        }
    }
}

#pragma mark -- OMImpressionDataDelegate
- (void)omImpressionData:(OMImpressionData * _Nullable)impressionData error:(NSError* _Nullable)error {
    NSString *impressionStr;
    if (error) {
        if (_impressionDataBlock) {
            NSString *errorStr = [NSString stringWithFormat:@"error code:%ld msg:%@", (long)[error code],[error localizedDescription]];
            _impressionDataBlock(OMImpressionDataEventFailed,errorStr);
        }
    } else {
        if (impressionData) {
            NSError *jsonErr = nil;
            NSDictionary *dict = @{
                @"impression_id":impressionData.impressionId,
                @"placement_id":impressionData.placementId,
                @"placement_name":impressionData.placementName,
                @"placement_ad_type":impressionData.placementAdType,
                @"scene_name":impressionData.sceneName,
                @"mediation_rule_id":impressionData.ruleId,
                @"mediation_rule_name":impressionData.ruleName,
                @"mediation_rule_type":impressionData.ruleType,
                @"mediation_rule_priority":impressionData.rulePriority,
                @"ab_group":impressionData.abGroup,
                @"instance_id":impressionData.instanceId,
                @"instance_name":impressionData.instanceName,
                @"instance_priority":impressionData.instancePriority,
                @"ad_network_id":impressionData.adNetworkId,
                @"ad_network_name":impressionData.adNetworkName,
                @"ad_network_unit_id":impressionData.adNetworkUnitId,
                @"precision":impressionData.precision,
                @"currency":impressionData.currency,
                @"revenue":impressionData.revenue,
                @"lifetime_value":impressionData.lifeTimeValue,
            };
            NSData *jsonData = [NSJSONSerialization dataWithJSONObject:dict options:NSJSONWritingPrettyPrinted error:&jsonErr];
            impressionStr = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
            
            if (_impressionDataBlock) {
                _impressionDataBlock(OMImpressionDataEventSuccess,impressionStr);
            }
        }
        
    }
}

@end

#endif
