#include "OpenMediationWrapper.h"

namespace Cocos {

    namespace OMPlugin {

        OpenMediationWrapper *OpenMediationWrapper::_instance = nullptr;
        OpenMediationWrapper* OpenMediationWrapper::getInstance() {
            if (nullptr == _instance) {
                _instance = new Cocos::OMPlugin::OpenMediationWrapper();
            }

            return _instance;
        }


        OpenMediationWrapper::OpenMediationWrapper():listener(nullptr) {

        }
        
        OpenMediationWrapper::~OpenMediationWrapper() {

        }
    

        void OpenMediationWrapper::setListener(OpenMediationWrapperListener *l) {
            listener = l;
        }
        
        OpenMediationWrapperListener *OpenMediationWrapper::getListener() {
            return listener;
        }
        
        void OpenMediationWrapper::removeListener() {
            listener = nullptr;
        }


        void OpenMediationWrapper::init(std::string &appKey) {
            nativeInit(appKey);
        }

        void OpenMediationWrapper::init(std::string &appKey, std::string &host) {
            nativeInit(appKey,host);
        }
    
        void OpenMediationWrapper::init(std::string &appKey, std::string &host, std::string &channel) {
            nativeInit(appKey,host,channel);
        }
        
        bool OpenMediationWrapper::isInitialized() {
            return nativeIsInitialized();
        }


        void OpenMediationWrapper::setIap(float count, std::string &currency) {
            nativeSetIap(count,currency);
        }

        void OpenMediationWrapper::debug(bool isDebug) {
            nativeDebug(isDebug);
        }

        void OpenMediationWrapper::setGDPRConsent(bool consent) {
            nativeSetGDPRConsent(consent);
        }

        void OpenMediationWrapper::setAgeRestricted(bool restricted) {
            nativeSetAgeRestricted(restricted);
        }
        

        void OpenMediationWrapper::setUserAge(int age) {
            nativeSetUserAge(age);
        }
        

        void OpenMediationWrapper::setUserGender(OpenMediationUserGender gender) {
            nativeSetUserGender(gender);
        }
        

        void OpenMediationWrapper::setUSPrivacyLimit(bool value) {
            nativeSetUSPrivacyLimit(value);
        }
        

        bool OpenMediationWrapper::getGDPRConsent() {
            return nativeGetGDPRConsent();
        }
        
    
        std::string OpenMediationWrapper::getVersion() {
            return COCOS_PLUGIN_VERSION;
        }

    
        void OpenMediationWrapper::setCustomTag(std::string &key, std::string &value) {
            nativeSetCustomTag(key,value);
        }

        void OpenMediationWrapper::setCustomTags(std::string &key, const std::vector<std::string>& values) {
            nativeSetCustomTags(key,values);
        }

        void OpenMediationWrapper::removeCustomTag(std::string &key) {
            nativeRemoveCustomTag(key);
        }
    

        std::string OpenMediationWrapper::getCustomTags() {
            return nativeGetCustomTags();
        }

        void OpenMediationWrapper::setUserId(std::string &userId) {
            nativeSetUserId(userId);
        }

        std::string OpenMediationWrapper::getUserId() {
            return nativeGetUserId();
        }

        // Interstitial
        bool OpenMediationWrapper::interstitialReady() {
            return nativeInterstitialReady();
        }
        

        void OpenMediationWrapper::showInterstitial() {
            nativeShowInterstitial();
        }


        void OpenMediationWrapper::showInterstitial(std::string &scene) {
            nativeShowInterstitial(scene);
        }
        


        // RewardedVideo

        bool OpenMediationWrapper::rewardedVideoReady() {
            return nativeRewardedVideoReady();
        }
        

        void OpenMediationWrapper::showRewardedVideo() {
            nativeShowRewardedVideo();
        }
        

        void OpenMediationWrapper::showRewardedVideo(std::string &scene) {
            nativeShowRewardedVideo(scene);
        }


        void OpenMediationWrapper::showRewardedVideo(std::string &scene, std::string &extraParams) {
            nativeShowRewardedVideo(scene,extraParams);
        }


        //Banner

        void OpenMediationWrapper::loadBanner(std::string &placementId, OpenMediationAdSize size, OpenMediationBannerPosition position) {
            nativeLoadBanner(placementId,size,position);
        }
        
        
        void OpenMediationWrapper::destroyBanner(std::string &placementId) {
            nativeDestroyBanner(placementId);
        }
        
        
        void OpenMediationWrapper::displayBanner(std::string &placementId) {
            nativeDisplayBanner(placementId);
        }
        

        void OpenMediationWrapper::hideBanner(std::string &placementId) {
            nativeHideBanner(placementId);
        }
        

        // PromotionAd
        bool OpenMediationWrapper::promotionAdReady() {
            return nativePromotionAdReady();
        }
        

        void OpenMediationWrapper::showPromotionAd(std::string &scene, int width, int height, float scaleX, float scaleY, float angle) {
            nativeShowPromotionAd(scene,width,height,scaleX,scaleY,angle);
        }


        void OpenMediationWrapper::hidePromotionAd() {
            nativeHidePromotionAd();
        }


        void OpenMediationWrapper::nativeInitCallBack(OpenMediationInitEvent event, const std::string &error) {
            if (nullptr != listener) {
                if (event == OpenMediationInitSuccess) {
                    listener->onInitSuccess();
                } else if (event == OpenMediationInitFailed) {

                    listener->onInitFailed(error);
                }
            }
        }
    
        
        void OpenMediationWrapper::nativeImpressionDataCallBack(OpenMediationImpressionDataEvent event, const std::string &value) {
            if (nullptr != listener) {
                if (event == OpenMediationImpressionData) {
                    listener->onImpressionData(value);
                } else if (event == OpenMediationImpressionDataError) {
                    listener->onImpressionDataError(value);
                }
            }
        }

        // Splash
        void OpenMediationWrapper::loadSplash(std::string &placementId) {
            nativeLoadSplash(placementId);
        }
    
        bool OpenMediationWrapper::splashAdReady(std::string &placementId) {
            return nativeSplashAdReady(placementId);
        }
    
        void OpenMediationWrapper::showSplash(std::string &placementId) {
            nativeShowSplash(placementId);
        }

        void OpenMediationWrapper::nativeAdEventCallBack(OpenMediationAdFormat adFormat, OpenMediationAdEvent event, bool available, const std::string &scene, const std::string &placementId, const std::string &error) {
            if (nullptr != listener) {
                switch(adFormat) {
                    case OpenMediationRewardedVideo:
                        {
                            switch(event) {
                                case OpenMediationAdAvailable:
                                    listener->onRewardedVideoAvailabilityChanged(available);
                                    break;
                                case OpenMediationAdShow:
                                    listener->onRewardedVideoShow(scene);
                                    break;
                                    
                                case OpenMediationAdShowFailed:
                                    listener->onRewardedVideoShowFailed(error);
                                    break;

                                case OpenMediationAdVideoStart:
                                    listener->onRewardedVideoStarted(scene);
                                    break;

                                case OpenMediationAdVideoEnd:
                                    listener->onRewardedVideoEnded(scene);
                                    break;
                                    
                                case OpenMediationAdClick:
                                    listener->onRewardedVideoClicked(scene);
                                    break;

                                case OpenMediationAdClose:
                                    listener->onRewardedVideoClosed(scene);
                                    break;
                                
                                case OpenMediationAdReceiveReward:
                                    listener->onRewardedVideoReceiveRewarded(scene);
                                    break;

                                default:
                                    break;
                            }
                        }
                        break;
                        
                    case OpenMediationInterstitial:
                        {
                            switch(event) {
                                case OpenMediationAdAvailable:
                                    listener->onInterstitialAvailabilityChanged(available);
                                    break;
                                case OpenMediationAdShow:
                                    listener->onInterstitialShow(scene);
                                    break;
                                    
                                case OpenMediationAdShowFailed:
                                    listener->onInterstitialShowFailed(error);
                                    break;
                                    
                                case OpenMediationAdClick:
                                    listener->onInterstitialClicked(scene);
                                    break;

                                case OpenMediationAdClose:
                                    listener->onInterstitialClosed(scene);
                                    break;

                                default:
                                    break;
                            }
                        }
                        break;

                    case OpenMediationPromotion:
                        {
                            switch(event) {
                                case OpenMediationAdAvailable:
                                    listener->onPromotionAdAvailabilityChanged(available);
                                    break;
                                case OpenMediationAdShow:
                                    listener->onPromotionAdShow(scene);
                                    break;
                                    
                                case OpenMediationAdShowFailed:
                                    listener->onPromotionAdShowFailed(error);
                                    break;
                                    
                                case OpenMediationAdClick:
                                    listener->onPromotionAdClicked(scene);
                                    break;

                                case OpenMediationAdClose:
                                    listener->onPromotionAdViewWillDisappear(scene);
                                    break;
                                    
                                default:
                                    break;
                            }
                        }
                        break;
                    
                    case OpenMediationSplash:
                        {
                            switch(event) {
                                case OpenMediationAdLoadSuccess:
                                    listener->onSplashLoadSuccess(placementId);
                                    break;

                                case OpenMediationAdLoadFailed:
                                    listener->onSplashLoadFailed(placementId, error);
                                    break;
                                    
                                case OpenMediationAdShow:
                                    listener->onSplashShow(placementId);
                                    break;
                                    
                                case OpenMediationAdShowFailed:
                                    listener->onSplashShowFailed(placementId, error);
                                    break;
                                    
                                case OpenMediationAdClick:
                                    listener->onSplashClicked(placementId);
                                    break;
                                    
                                case OpenMediationAdClose:
                                    listener->onSplashClosed(placementId);
                                    break;
                                    
                                default:
                                    break;
                            }
                        }
                        break;

                    case OpenMediationBanner:
	                	{
	                		switch(event) {
	                			case OpenMediationAdLoadSuccess:
	                				listener->onBannerLoadSuccess(placementId);
	                				break;

	                			case OpenMediationAdLoadFailed:
	                				listener->onBannerLoadFailed(placementId, error);
	                				break;

	                			case OpenMediationAdClick:
	                				listener->onBannerClicked(placementId);
	                				break;

	                			default:
	                				break;
	                		}
	                	}
	                	break;
                    
                    default:
                        break;
                }
            }
        }
    }
}
