
#ifndef __OpenMediationWrapper_h__
#define __OpenMediationWrapper_h__

#include <string>
#include <map>

#define PLUGIN_LOG(...) cocos2d::log(__VA_ARGS__)

namespace Cocos {

    #define JAVA_CLASS_OPENMEDIATION    "com/openmediation/sdk/cocos/OmBridge"
    #define COCOS_PLUGIN_VERSION "1.3.0"

    namespace OMPlugin {

            typedef enum {

                OpenMediationInitSuccess = 0,
                OpenMediationInitFailed

            } OpenMediationInitEvent;


            typedef enum {

                OpenMediationUserGenderUnknown = 0,

                OpenMediationUserGenderMale,

                OpenMediationUserGenderFemale,

            } OpenMediationUserGender;

            typedef enum {

                OpenMediationBanner = (1 << 0),

                OpenMediationNative = (1 << 1),

                OpenMediationRewardedVideo = (1 << 2),

                OpenMediationInterstitial = (1 << 3),

                OpenMediationSplash = (1 << 4),

                OpenMediationPromotion = (1 << 5),

            } OpenMediationAdFormat;


            typedef enum {

                OpenMediationAdAvailable = 0,

                OpenMediationAdLoadSuccess,

                OpenMediationAdLoadFailed,

                OpenMediationAdShow,

                OpenMediationAdShowFailed,

                OpenMediationAdVideoStart,

                OpenMediationAdVideoEnd,

                OpenMediationAdClick,

                OpenMediationAdClose,

                OpenMediationAdReceiveReward,

            } OpenMediationAdEvent;


            typedef enum {

                OpenMediationAdSizeBanner = 0,

                OpenMediationAdSizeMediumRectangle,

                OpenMediationAdSizeLeaderboard,

                OpenMediationAdSizeSmart,

            } OpenMediationAdSize;

            typedef enum {

                OpenMediationBannerPositionBottom = 0,

                OpenMediationBannerPositionTop,

            } OpenMediationBannerPosition;

            typedef enum {
                OpenMediationImpressionData = 0,
                
                OpenMediationImpressionDataError,
            
            } OpenMediationImpressionDataEvent;
            
            class OpenMediationWrapperListener{
            public:

                // Init
                virtual void onInitSuccess() = 0;

                virtual void onInitFailed(const std::string &error) = 0;

                // RewardedVideo
                virtual void onRewardedVideoAvailabilityChanged(bool available) = 0;

                virtual void onRewardedVideoShow(const std::string &scene) = 0;

                virtual void onRewardedVideoShowFailed(const std::string error) = 0;

                virtual void onRewardedVideoClosed(const std::string &scene) = 0;

                virtual void onRewardedVideoStarted(const std::string &scene) = 0;

                virtual void onRewardedVideoEnded(const std::string &scene) = 0;

                virtual void onRewardedVideoReceiveRewarded(const std::string &scene) = 0;

                virtual void onRewardedVideoClicked(const std::string &scene) = 0;

                // Interstitial
                virtual void onInterstitialAvailabilityChanged(bool available) = 0;

                virtual void onInterstitialShow(const std::string &scene) = 0;

                virtual void onInterstitialShowFailed(const std::string error) = 0;

                virtual void onInterstitialClosed(const std::string &scene) = 0;

                virtual void onInterstitialClicked(const std::string &scene) = 0;

                // PromotionAd
                virtual void onPromotionAdAvailabilityChanged(bool available) = 0;

                virtual void onPromotionAdShow(const std::string &scene) = 0;

                virtual void onPromotionAdShowFailed(const std::string &error) = 0;

                virtual void onPromotionAdViewWillDisappear(const std::string &scene) = 0;

                virtual void onPromotionAdClicked(const std::string &scene) = 0;


                //Banner

                virtual void onBannerLoadSuccess(const std::string placementId) = 0;

                virtual void onBannerLoadFailed(const std::string placementId, const std::string error) = 0;

                virtual void onBannerClicked(const std::string placementId) = 0;
                
                // Splash
                virtual void onSplashLoadSuccess(const std::string placementId) = 0;

                virtual void onSplashLoadFailed(const std::string placementId, const std::string error) = 0;
                
                virtual void onSplashShow(const std::string placementId) = 0;

                virtual void onSplashShowFailed(const std::string placementId, const std::string error) = 0;

                virtual void onSplashClicked(const std::string placementId) = 0;

                virtual void onSplashClosed(const std::string placementId) = 0;

                //ImpressionData
                virtual void onImpressionDataError(const std::string error) = 0;

                virtual void onImpressionData(const std::string impressionData) = 0;
                
            };
            
            class OpenMediationWrapper {
            public:
                OpenMediationWrapper();
                ~OpenMediationWrapper();

                static OpenMediationWrapper *getInstance();

                void setListener(OpenMediationWrapperListener *listener);
                OpenMediationWrapperListener *getListener();
                void removeListener();

                void init(std::string &appKey);
                void init(std::string &appKey, std::string &host);
                void init(std::string &appKey, std::string &host, std::string &channel);
                bool isInitialized();


                void setIap(float count, std::string &currency);

                void debug(bool isDebug);

                void setGDPRConsent(bool consent);

                void setAgeRestricted(bool restricted);

                void setUserAge(int age);

                void setUserGender(OpenMediationUserGender gender);

                void setUSPrivacyLimit(bool value);

                bool getGDPRConsent();
                
                std::string getVersion();
                
                void setCustomTag(std::string &key, std::string &value);

                void setCustomTags(std::string &key, const std::vector<std::string>& values);
                
                void removeCustomTag(std::string &key);
                
                std::string getCustomTags();

                void setUserId(std::string &userId);

                std::string getUserId();

                // Interstitial
                bool interstitialReady();

                void showInterstitial();

                void showInterstitial(std::string &scene);


                // RewardedVideo

                bool rewardedVideoReady();

                void showRewardedVideo();

                void showRewardedVideo(std::string &scene);

                void showRewardedVideo(std::string &scene, std::string &extraParams);

                //Banner

                void loadBanner(std::string &placementId, OpenMediationAdSize size, OpenMediationBannerPosition position);

                void destroyBanner(std::string &placementId);

                void displayBanner(std::string &placementId);

                void hideBanner(std::string &placementId);

                // PromotionAd
                bool promotionAdReady();

                void showPromotionAd(std::string &scene, int width, int height, float scaleX, float scaleY, float angle);

                void hidePromotionAd();

                // Splash
                void loadSplash(std::string &placementId);

                bool splashAdReady(std::string &placementId);

                void showSplash(std::string &placementId);


            public:

                void nativeInit(std::string &appKey);
                void nativeInit(std::string &appKey, std::string &host);
                void nativeInit(std::string &appKey, std::string &host, std::string &channel);
                bool nativeIsInitialized();

                // void setGDPR(bool enabled);

                // AdColonyAdStatus getGDPR();


                void nativeSetIap(float count, std::string &currency);

                void nativeDebug(bool isDebug);

                void nativeSetGDPRConsent(bool consent);

                void nativeSetAgeRestricted(bool restricted);

                void nativeSetUserAge(int age);

                void nativeSetUserGender(OpenMediationUserGender gender);

                void nativeSetUSPrivacyLimit(bool value);

                bool nativeGetGDPRConsent();

                void nativeSetCustomTag(std::string &key, std::string &value);

                void nativeSetCustomTags(std::string &key, const std::vector<std::string>& values);

                void nativeRemoveCustomTag(std::string &key);

                std::string nativeGetCustomTags();

                void nativeSetUserId(std::string &userId);

                std::string nativeGetUserId();

                // Interstitial
                bool nativeInterstitialReady();

                void nativeShowInterstitial();

                void nativeShowInterstitial(std::string &scene);


                // RewardedVideo

                bool nativeRewardedVideoReady();

                void nativeShowRewardedVideo();

                void nativeShowRewardedVideo(std::string &scene);

                void nativeShowRewardedVideo(std::string &scene, std::string &extraParams);


                //Banner

                void nativeLoadBanner(std::string &placementId, OpenMediationAdSize size, OpenMediationBannerPosition position);
                
                void nativeDestroyBanner(std::string &placementId);
                
                void nativeDisplayBanner(std::string &placementId);
        
                void nativeHideBanner(std::string &placementId);

                // PromotionAd
                bool nativePromotionAdReady();

                void nativeShowPromotionAd(std::string &scene, int width, int height, float scaleX, float scaleY, float angle);

                void nativeHidePromotionAd();
                
                void nativeSetAdBlock();

                void nativeInitCallBack(OpenMediationInitEvent event, const std::string &error);

                void nativeAdEventCallBack(OpenMediationAdFormat adFormat, OpenMediationAdEvent event, bool available, const std::string &scene, const std::string &placementId, const std::string &error);

                void nativeImpressionDataCallBack(OpenMediationImpressionDataEvent event, const std:: string &value);
                
                // Splash
                void nativeLoadSplash(std::string &placementId);
                                
                bool nativeSplashAdReady(std::string &placementId);
                
                void nativeShowSplash(std::string &placementId);

            protected:


            protected:
                OpenMediationWrapperListener *listener;

            private:
                static OpenMediationWrapper *_instance;

            };
    }
}

#endif
